@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009, 2010  Akira Terasaki
+ * Copyright (c) 2009-2012  Akira Terasaki
  * このファイルは同梱されている License.txt に定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -20,11 +20,12 @@ public class Version {
 	private Version(){}
 	/**
 	 * Paraseleneのバージョン番号を表す文字列。
-	 * 2.2.2.32
+	 * 2.3.1.0
 	 */
-	public static final String VERSION = "2.2.2.32";
+	public static final String VERSION = "2.3.1.0";
 	private static final String	SITE = " (http://paraselene.sourceforge.jp/)";
 	private static final String	TITLE = "Paraselene2 " + VERSION + SITE;
+	private static final String COPY = "Copyright 2009-2012 Akira Terasaki";
 	private static final String[] CLASS_PATH = new String[] {
 		"sun.boot.class.path",
 		"java.ext.dirs",
@@ -60,7 +61,7 @@ public class Version {
 	 * @return 著作権表示。
 	 */
 	public static String getCopyRight() {
-		return "Copyright 2009 Akira Terasaki";
+		return COPY;
 	}
 
 	private static boolean isHit( File f ) {
@@ -116,9 +117,7 @@ public class Version {
 			file = new File( path );
 			return file;
 		}
-		catch( Exception e ) {
-			e.printStackTrace();
-		}
+		catch( Exception e ) {}
 		for ( int i = 0; i < CLASS_PATH.length; i++ ) {
 			File	ret = seekJar( CLASS_PATH[i] );
 			if ( ret != null )	return ret;
