@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009, 2010  Akira Terasaki
+ * Copyright (c) 2009-2011  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -16,6 +16,7 @@ import paraselene.tag.*;
 import paraselene.tag.list.*;
 import paraselene.tag.form.*;
 import paraselene.tag.table.*;
+import paraselene.tag.ruby.*;
 import paraselene.dyna.*;
 import paraselene.css.*;
 
@@ -274,18 +275,123 @@ public class TagMap {
 			}
 		},
 		{ "password", paraselene.tag.form.SingleTextBox.class,
-			"new SingleTextBox(true)",
+			"new SingleTextBox(SingleTextBox.Type.PASSWORD)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
-					return setAttr( new SingleTextBox( true ), attr );
+					return setAttr( new SingleTextBox( SingleTextBox.Type.PASSWORD ), attr );
 				}
 			}
 		},
 		{ "text", paraselene.tag.form.SingleTextBox.class,
-			"new SingleTextBox(false)",
+			"new SingleTextBox(SingleTextBox.Type.TEXT)",
 			new DynaTag() {
 				Tag getTag( String name, ArrayList<Attribute> attr ) {
-					return setAttr( new SingleTextBox( false ), attr );
+					return setAttr( new SingleTextBox( SingleTextBox.Type.TEXT ), attr );
+				}
+			}
+		},
+	// HTML5 input
+		{ "search", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.SEARCH)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.SEARCH ), attr );
+				}
+			}
+		},
+		{ "tel", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.TELEPHONE)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.TELEPHONE ), attr );
+				}
+			}
+		},
+		{ "url", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.URL)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.URL ), attr );
+				}
+			}
+		},
+		{ "email", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.EMAIL)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.EMAIL ), attr );
+				}
+			}
+		},
+		{ "datetime", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.DATE_AND_TIME)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.DATE_AND_TIME ), attr );
+				}
+			}
+		},
+		{ "date", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.DATE)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.DATE ), attr );
+				}
+			}
+		},
+		{ "month", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.MONTH)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.MONTH ), attr );
+				}
+			}
+		},
+		{ "week", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.WEEK)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.WEEK ), attr );
+				}
+			}
+		},
+		{ "time", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.TIME)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.TIME ), attr );
+				}
+			}
+		},
+		{ "datetime-local", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.LOCAL_DATE_AND_TIME)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.LOCAL_DATE_AND_TIME ), attr );
+				}
+			}
+		},
+		{ "number", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.NUMBER)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.NUMBER ), attr );
+				}
+			}
+		},
+		{ "range", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.RANGE)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.RANGE ), attr );
+				}
+			}
+		},
+		{ "color", paraselene.tag.form.SingleTextBox.class,
+			"new SingleTextBox(SingleTextBox.Type.COLOR)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SingleTextBox( SingleTextBox.Type.COLOR ), attr );
 				}
 			}
 		},
@@ -315,6 +421,14 @@ public class TagMap {
 				}
 			}
 		},
+		{ "button", paraselene.tag.form.GPButton.class,
+			"new GPButton()",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new GPButton(), attr );
+				}
+			}
+		},
 		{ "select", paraselene.tag.form.Select.class,
 			"new Select()",
 			new DynaTag() {
@@ -323,6 +437,14 @@ public class TagMap {
 				}
 			}
 		},
+		{ "optgroup", paraselene.tag.form.SelectGroup.class,
+			"new SelectGroup()",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new SelectGroup(), attr );
+				}
+			}
+		},
 		{ "option", paraselene.tag.form.SelectItem.class,
 			"new SelectItem()",
 			new DynaTag() {
@@ -411,6 +533,54 @@ public class TagMap {
 				}
 			}
 		},
+		{ "ruby", paraselene.tag.ruby.Ruby.class,
+			"new Ruby()",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new Ruby(), attr );
+				}
+			}
+		},
+		{ "rb", paraselene.tag.ruby.RubyText.class,
+			"new RubyText(RubyText.Type.BASE)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new RubyText( RubyText.Type.BASE ), attr );
+				}
+			}
+		},
+		{ "rbc", paraselene.tag.ruby.RubyText.class,
+			"new RubyText(RubyText.Type.BASE,new RubyText[0])",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new RubyText( RubyText.Type.BASE, new RubyText[0] ), attr );
+				}
+			}
+		},
+		{ "rt", paraselene.tag.ruby.RubyText.class,
+			"new RubyText(RubyText.Type.TEXT)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new RubyText( RubyText.Type.TEXT ), attr );
+				}
+			}
+		},
+		{ "rtc", paraselene.tag.ruby.RubyText.class,
+			"new RubyText(RubyText.Type.TEXT,new RubyText[0])",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new RubyText( RubyText.Type.TEXT, new RubyText[0] ), attr );
+				}
+			}
+		},
+		{ "rp", paraselene.tag.ruby.RubyText.class,
+			"new RubyText(RubyText.Type.PARENTHESES)",
+			new DynaTag() {
+				Tag getTag( String name, ArrayList<Attribute> attr ) {
+					return setAttr( new RubyText( RubyText.Type.PARENTHESES ), attr );
+				}
+			}
+		},
 	};
 
 	static {
