@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009, 2010  Akira Terasaki
+ * Copyright (c) 2009-2011  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -14,7 +14,7 @@ import paraselene.*;
  */
 public class Feedback extends Forward implements AjaxForward {
 	private static final long serialVersionUID = 2L;
-	private Page target;
+	Page target;
 
 	/**
 	 * コンストラクタ。
@@ -36,7 +36,7 @@ public class Feedback extends Forward implements AjaxForward {
 		target = page.getParentPage();
 	}
 
-	Page getPage( Page from ) {
+	Page getPage( PageFactory pf, Page from ) {
 		Option.trace( "Feedback#getPage -> %s", target.getUniqueKey() );
 		return target;
 	}
@@ -59,19 +59,21 @@ public class Feedback extends Forward implements AjaxForward {
 
 	public String toString() {
 		StringBuilder	buf = new StringBuilder( "Feedback:" );
-		buf = buf.append( target.getID().toString() );
-		buf = buf.append( "/" ).append( target.getUniqueKey() );
+		if ( target != null ) {
+			buf = buf.append( target.getID() );
+			buf = buf.append( "/" ).append( target.getUniqueKey() );
+		}
 		return buf.toString();
 	}
 
-	OutputCall makeOutput( int nk, Page lp, RequestParameter r, SessionData sd ) {
+	OutputCall makeOutput( PageFactory pf, int nk, Page lp, RequestParameter r, SessionData sd ) {
 		Option.trace( "new OutputCall <- %s", this );
-		return new OutputCall( nk, lp, this, r, sd );
+		return new OutputCall( pf, nk, lp, this, r, sd );
 	}
 
-	CometCall makeCometCall( History h ) {
+	CometCall makeCometCall( PageFactory pf, History h ) {
 		Option.trace( "new CometCall <- %s", this );
-		return new CometCall( h, this );
+		return new CometCall( pf, h, this );
 	}
 }
 
