@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009, 2010  Akira Terasaki
+ * Copyright (c) 2009-2012  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -18,6 +18,7 @@ import paraselene.*;
  */
 public class Forward implements Serializable {
 	private static final long serialVersionUID = 212L;
+	boolean weak_f = false;
 	private int	error = 0;
 	private transient URI	redirect_uri = null;
 	PageID	out_page = null;
@@ -40,8 +41,9 @@ public class Forward implements Serializable {
 		return is_refresh();
 	}
 
+	private boolean leave_f = true;
 	boolean isLeave() {
-		return error > 0 || redirect_uri != null;
+		return leave_f && (error > 0 || redirect_uri != null);
 	}
 
 	/**
@@ -64,10 +66,9 @@ public class Forward implements Serializable {
 	 */
 	public Forward( int stat ) {
 		error = stat;
+		if ( (stat % 100) <= 2 )	return;
 		session_clear_f = true;
-		if ( stat == 403 ) {
-			ste403 = Thread.currentThread().getStackTrace();
-		}
+		ste403 = Thread.currentThread().getStackTrace();
 	}
 
 	/**
@@ -79,6 +80,10 @@ public class Forward implements Serializable {
 		redirect_uri = uri;
 		session_clear_f = session_off;
 	}
+	Forward( URI uri, boolean session_off, boolean lf ) {
+		this( uri, session_off );
+		leave_f = lf;
+	}
 
 	/**
 	 * コンストラクタ。ページへ遷移。
@@ -88,8 +93,12 @@ public class Forward implements Serializable {
 	 * @param session_off true:セッションを解放する、false:セッションを維持する
 	 */
 	public Forward( PageID id, boolean history_use, boolean session_off ) {
-		if ( !(this instanceof Closure) && !(this instanceof Feedback) ) {
-			id = id.getPageFactory().getAjaxSurrogatePage( id );
+		this( SandBox.getPageFactory(), id, history_use, session_off );
+	}
+
+	Forward( PageFactory pf, PageID id, boolean history_use, boolean session_off ) {
+		if ( pf != null && !(this instanceof Closure) && !(this instanceof Feedback) ) {
+			id = pf.getAjaxSurrogatePage( id );
 		}
 		out_page = id;
 		history_f = history_use;
@@ -120,11 +129,115 @@ public class Forward implements Serializable {
 		if ( af.length == 0 ) {
 			af = new AjaxForward[]{ new NOP() };
 		}
+		else {
+			for ( AjaxForward a: af ) {
+				for ( Cookie c: ((Forward)a).out_cookie ) {
+					out_cookie.add( c );
+				}
+			}
+		}
 		ajax = af;
 		if ( SandBox.isCurrentDaemon() )	return;
 		initTrim( SandBox.getCurrentRequestParameter().getHistory() );
 	}
 
+	/**
+	 * 複数の Forward の統合と取捨。
+	 * 指定された Forward を１つにします。戻り値は次のようになります。
+	 * <ol>
+	 * <li>引数のうち、AjaxForward を持たない単独の Forward があれば、最初に登場した
+	 * それを返します。
+	 * <li>引数のうち、AjaxForward を持つ Forward があれば、設定されている全ての
+	 * AjaxForward を取得します。<br>
+	 * <li>引数のうち、直接 AjaxForward があれば、それらを取得します。
+	 * <li>取得した AjaxForward が0件であれば、NOPインスタンスを返します。
+	 * <li>取得した AjaxForward が1件であれば、それを返します。
+	 * <li>取得した AjaxForward が複数件であれば、それらを格納した Forward を
+	 * 返します。
+	 * </ol>
+	 * @param fw 統合対象。
+	 * @return 統合した Forward。
+	 */
+	public static Forward merge( Forward ... fw ) {
+		if ( fw.length == 0 )	return new NOP();
+		if ( fw.length == 1 )	return fw[0];
+		ArrayList<AjaxForward>	list = new ArrayList<AjaxForward>();
+		for ( int i = 0; i < fw.length; i++ ) {
+			if ( fw[i].weak_f )	continue;
+			if ( fw[i] == null )	continue;
+			if ( fw[i] instanceof NOP )	continue;
+			if ( fw[i] instanceof AjaxForward ) {
+				list.add( (AjaxForward)fw[i] );
+				Option.trace( "add %s", fw[i] );
+				continue;
+			}
+			if ( fw[i].isAjaxHold() ) {
+				for ( AjaxForward af: fw[i].getAjaxForward( true ) ) {
+					if ( af == null )	continue;
+					list.add( af );
+					Option.trace( "add %s", af );
+				}
+				continue;
+			}
+			return fw[i];
+		}
+		switch( list.size() ) {
+		case 0:	return new NOP();
+		case 1: return (Forward)list.get( 0 );
+		}
+		HashMap<Integer, ArrayList<AjaxForward>>	pf = new HashMap<Integer, ArrayList<AjaxForward>>();
+		HashMap<Integer, AjaxForward>	closure = new HashMap<Integer, AjaxForward>();
+		ArrayList<AjaxForward>	etc = new ArrayList<AjaxForward>();
+		for ( AjaxForward af: list ) {
+			PageID	pid = af.getPageID();
+			if ( af instanceof Closure ) {
+				pid = ((Closure)af).close_page.getID();
+			}
+			else if ( af instanceof Feedback ) {
+				pid = ((Feedback)af).target.getID();
+			}
+			if ( pid == null || af instanceof JavaScript ) {
+				etc.add( af );
+				continue;
+			}
+			int	id = pid.getID();
+			if ( af instanceof Closure ) {
+				Option.trace( "Closure %s", pid );
+				pf.remove( id );
+				if ( closure.get( id ) instanceof EphemeralClosure )	continue;
+				closure.put( id, af );
+				continue;
+			}
+			if ( closure.get( id ) != null ) {
+				Option.trace( "not Closure %s continue", pid );
+				continue;
+			}
+			ArrayList<AjaxForward>	l = pf.get( id );
+			if ( l == null ) {
+				l = new ArrayList<AjaxForward>();
+				pf.put( id, l );
+			}
+			Option.trace( "not Closure %s add", pid );
+			l.add( af );
+		}
+		list = new ArrayList<AjaxForward>();
+		for ( int id: closure.keySet() )	list.add( closure.get( id ) );
+		for ( int id: pf.keySet() ) {
+			for( AjaxForward af: pf.get( id ) )	list.add( af );
+		}
+		for ( AjaxForward af: etc )	list.add( af );
+		return new Forward( list.toArray( new AjaxForward[0] ) );
+	}
+
+	boolean isAjaxHold() {
+		return ajax != null;
+	}
+
+	boolean isAjax() {
+		if ( this instanceof AjaxForward )	return true;
+		return isAjaxHold();
+	}
+
 	void initTrim( History hist ) {
 		if ( ajax == null )	return;
 		ajax = trim( ajax, hist, true );
@@ -146,7 +259,7 @@ public class Forward implements Serializable {
 		if ( af.length < 1 )	return af;
 		if ( hist == null )	return null;
 		Page[]	page = hist.getBrowsingPage();
-		Popup.Type	ptype = Popup.Type.MODELESS;
+		DialogType	ptype = Popup.Type.MODELESS;
 		if ( Popup.isModelessAlert( ptype, hist, false ) ) {
 			ptype = Popup.Type.MODAL;
 		}
@@ -197,7 +310,7 @@ public class Forward implements Serializable {
 		return ret;
 	}
 
-	Page getPage( Page from ) {
+	Page getPage( PageFactory pf, Page from ) {
 		if ( out_page == null ) {
 			Option.trace( "Forward#getPage -> null" );
 			return null;
@@ -208,7 +321,7 @@ public class Forward implements Serializable {
 				return from;
 			}
 		}
-		Page	p = out_page.getPageFactory().getPage( out_page.getID() );
+		Page	p = pf.getPage( out_page.getID() );
 		Option.trace( "Forward#getPage -> %s", p.getUniqueKey() );
 		return p;
 	}
@@ -245,13 +358,15 @@ public class Forward implements Serializable {
 		if ( done_f )	return;
 		done_f = true;
 		if ( error != 0 ) {
-			if ( error == 403 ) {
-				System.out.println( "================ 403 Forward  ==================" );
-				System.out.println( new java.util.Date().toString() );
-				for ( int i = 0; i < ste403.length; i++ ) {
-					System.out.println( ste403[i].toString() );
+			if ( ste403 != null ) {
+				StringBuilder	buf = new StringBuilder();
+				for ( StackTraceElement s: ste403 ) {
+					buf = buf.append( s.toString() ).append( "\n" );
 				}
-				System.out.println( "================================================" );
+				Option.trace( "%d Forward\n------------------------------------------------------\n%s", error, buf.toString() );
+			}
+			else {
+				res.setContentType( "text/plain" );
 			}
 			ErrorStatus.send( res, error );
 			return;
@@ -287,14 +402,14 @@ public class Forward implements Serializable {
 		return buf.toString();
 	}
 
-	OutputCall makeOutput( int nk, Page lp, RequestParameter r, SessionData sd ) {
+	OutputCall makeOutput( PageFactory pf, int nk, Page lp, RequestParameter r, SessionData sd ) {
 		Option.trace( "new LocationOutput <- %s", this );
-		return new LocationOutput( nk, lp, this, r, sd );
+		return new LocationOutput( pf, nk, lp, this, r, sd );
 	}
 
-	CometCall makeCometCall( History h ) {
+	CometCall makeCometCall( PageFactory pf, History h ) {
 		Option.trace( "new LocationCometCall <- %s", this );
-		return new LocationCometCall( h, this );
+		return new LocationCometCall( pf, h, this );
 	}
 }
 
