@@ -23,7 +23,7 @@ public class HistorySet implements Serializable {
 	private HashMap<Integer,History>	hist = new HashMap<Integer,History>();
 	private String session_id;
 
-	void replaceSession( String sid ) {
+	synchronized void replaceSession( String sid ) {
 		session_id = sid;
 		for ( History	h : hist.values() ) {
 			h.session_id = sid;
