@@ -42,5 +42,11 @@ public interface ServerInformation extends Serializable {
 	 * @return ページファクトリー。
 	 */
 	PageFactory getPageFactory();
+	/**
+	 * 分離セッションIDの取得。
+	 * @return !null:セッションID、null:通常セッション。
+	 */
+	String getMultiStageSessionID();
+
 }
 
