@@ -1,6 +1,6 @@
 /*
  * Paraselene
- * Copyright (c) 2009, 2010  Akira Terasaki
+ * Copyright (c) 2009-2012  Akira Terasaki
  * このファイルは同梱されているLicense.txtに定めた条件に同意できる場合にのみ
  * 利用可能です。
  */
@@ -354,9 +354,10 @@ public class Tag extends HTMLPart {
 			if ( "id".equals( k ) )	continue;
 			tag.setAttribute( attr.get( k ).getReplica() );
 		}
-		int	size = doc.size();
-		for ( int i = 0; i < size; i++ ) {
-			tag.addHTMLPart( doc.get( i ).getReplica() );
+		synchronized( doc ) {
+			for ( HTMLPart p: doc ) {
+				tag.addHTMLPart( p.getReplica() );
+			}
 		}
 		return tag;
 	}
@@ -572,7 +573,9 @@ public class Tag extends HTMLPart {
 	 * @param d 文書。
 	 */
 	public void addHTMLPart( HTMLPart d ) {
-		addHTMLPart( doc.size(), d );
+		synchronized( doc ) {
+			addHTMLPart( doc.size(), d );
+		}
 	}
 
 	/**
@@ -665,24 +668,34 @@ public class Tag extends HTMLPart {
 	public void addHTMLPart( int idx, HTMLPart d ) {
 		if ( this instanceof VirtualTag || d instanceof VirtualTag )	return;
 		setModify();
-		doc.add( idx, d );
+		synchronized( doc ) {
+			doc.add( idx, d );
+		}
+		d.setAssignedTag( this );
 		if ( d instanceof Tag ) {
 			Tag tag = (Tag)d;
 			int	cnt = doc.size();
-			idx = 0;
-			for ( int i = 0; i < cnt; i++ ) {
-				HTMLPart	p = doc.get( i );
-				if ( p == d )	break;
-				if ( p instanceof Tag ) {
-					idx++;
+			Tag	last_tag = null;
+			synchronized( doc ) {
+				for ( HTMLPart p: doc ) {
+					if ( p == d )	break;
+					if ( p instanceof Tag )	last_tag = (Tag)p;
 				}
 			}
-			tag_only.add( idx, tag );
+			if ( last_tag != null ) {
+				synchronized( tag_only ) {
+					idx = tag_only.indexOf( last_tag );
+					if ( idx >= 0 ) {
+						tag_only.add( idx + 1, tag );
+					}
+					else	tag_only.add( tag );
+				}
+			}
+			else	tag_only.add( tag );
 			getAssignedPage().nameTag( tag );
 			if ( tag.getAttribute( "name" ) != null )	makeID();
 			setLocation();
 		}
-		d.setAssignedTag( this );
 	}
 
 	/**
@@ -805,7 +818,9 @@ public class Tag extends HTMLPart {
 		setModify();
 		if ( part instanceof Tag ) {
 			Tag	tag = (Tag)part;
-			tag_only.remove( tag );
+			synchronized( tag_only ) {
+				tag_only.remove( tag );
+			}
 			getAssignedPage().removeNameEntry( tag );
 		}
 		part.setAssignedTag( null );
@@ -842,7 +857,11 @@ public class Tag extends HTMLPart {
 	 */
 	public int indexOf( HTMLPart part ) {
 		if ( part == null ) return -1;
-		return doc.indexOf( part );
+		int	size = doc.size();
+		for ( int i = 0; i < size; i++ ) {
+			if ( doc.get( i ) == part )	return i;
+		}
+		return -1;
 	}
 
 	/**
@@ -1031,6 +1050,25 @@ public class Tag extends HTMLPart {
 		setValueAttribute( v );
 	}
 
+	/**
+	 * 値の設定。<br>
+	 * setValueString( tag.getValueString() )と等価です。
+	 * @param tag 設定値。
+	 */
+	public void setValueTag( Tag tag ) {
+		setValueString( tag.getValueString() );
+	}
+
+	/**
+	 * 値の設定。<br>
+	 * 閉じタグを持たないものであれば、value属性値に設定します。<br>
+	 * 閉じタグを持つものであれば、タグ内部に持たせます。
+	 * @param n 設定値。
+	 */
+	public void setValueNumber( Number n ) {
+		setValueString( n.toString() );
+	}
+
 	/**
 	 * 値を持っているか？
 	 * @return true:空文字列かnull値である、false:値を持っている。
@@ -1047,7 +1085,9 @@ public class Tag extends HTMLPart {
 	 * @return タグ。無ければ0個の配列を返す。
 	 */
 	public Tag[] getTagArray() {
-		return tag_only.toArray( new Tag[0] );
+		synchronized( tag_only ) {
+			return tag_only.toArray( new Tag[0] );
+		}
 	}
 
 	/**
@@ -1121,13 +1161,18 @@ public class Tag extends HTMLPart {
 	 * @return true:持っている、false:持っていない。
 	 */
 	public boolean isInner( HTMLPart target ) {
-		return getDirectInnerTag( target ) != null;
+		while( true ) {
+			target = getDirectInnerTag( target );
+			if ( target == null )	return false;
+			if ( target == this )	return true;
+		}
 	}
 
 	/**
-	 * 指定インスタンスを直接保持しているタグを返す。
+	 * 指定インスタンスを直接保持しているタグを返す。<br>
+	 * ただし、thisの内部のタグしか検索しません。
 	 * @param target 検索対象。
-	 * @return 直接保持しているタグ。無ければnull。
+	 * @return 直接保持しているタグ。無ければnull。targetがthisの場合もnullです。
 	 */
 	public Tag getDirectInnerTag( HTMLPart target ) {
 		if ( this == target )	return null;
@@ -1324,8 +1369,9 @@ public class Tag extends HTMLPart {
 	 * &quot;  alert( 'CLICK' );&quot;,
 	 * &quot;}&quot;
 	 * </pre>のような文字列群です。各文字列には改行が追加されます。
+	 * @return 無名関数を格納した変数名を返します。
 	 */
-	public void setAnonymousEventHandler( String triger, String ... function ) {
+	public String setAnonymousEventHandler( String triger, String ... function ) {
 		Page	page = getAssignedPage();
 		String	id = makeID();
 		StringBuilder	script = new StringBuilder();
@@ -1347,6 +1393,7 @@ public class Tag extends HTMLPart {
 		buf = buf.append( ";" );
 		page.addOnLoadScript( buf.toString() );
 		setEventHandler( triger, ano );
+		return ano;
 	}
 
 	/**
@@ -1372,7 +1419,40 @@ public class Tag extends HTMLPart {
 			public void setModify() {return;}
 		};
 		tag.setAttribute( new Attribute( "type", "text/javascript" ) );
-		tag.addHTMLPart( new paraselene.ajax.data.JavaScript( script ) );
+		tag.addHTMLPart( new paraselene.ajax.data.JavaScript( false, script ) );
+		return tag;
+	}
+
+	private static class Style extends PlainText {
+		private static final long serialVersionUID = 2L;
+		private Style(){}
+		public Style( String ... style ) {
+			super( "" );
+			appendWithCRLF( style );
+		}
+		public String toString( StringMode mode ) {
+			StringBuilder	buf = new StringBuilder( "/*<![CDATA[*/\r\n" );
+			buf = buf.append( toString() );
+			buf = buf.append( "/*]]>*/" );
+			return buf.toString();
+		}
+		public String toString() {
+			return super.toString( StringMode.PLAIN );
+		}
+	}
+
+	/**
+	 * スタイルシートを安全に記述するための style タグを生成します。<br>
+	 * このタグの内容を変更しても、Feedback として通知されないので注意して下さい。
+	 * @param style スタイルシート。各引数には改行が挟まれて出力されます。
+	 * @return style タグ。
+	 */
+	public static Tag makeStyle( String ... style ) {
+		Tag	tag = new Tag( "style", false ) {
+			public void setModify() {return;}
+		};
+		tag.setAttribute( new Attribute( "type", "text/css" ) );
+		tag.addHTMLPart( new Style( style ) );
 		return tag;
 	}
 
