@@ -388,10 +388,8 @@ public class Table extends Tag {
 			Column[]	clm = l[idx - 1].getColumnArray();
 			for ( int i = 0; i < clm.length; i++ ) {
 				int row = clm[i].getRowspan();
-				row--;
-				if ( row > 1 ) {
-					clm[i].setRowspan( row );
-				}
+				if ( row < 1 )	continue;
+				clm[i].setRowspan( row - 1 );
 			}
 		}
 		removeHTMLPart( l[idx] );
